/*
 * This file is part of SkunkDB
 * Copyright (c) 2002 Gianni Tedesco
 * Released under the terms of the GNU GPL version 2
*/
#ifndef __AVL_HEADER_INCLUDED__
#define __AVL_HEADER_INCLUDED__

typedef u_int32_t avl_ofs_t;
typedef u_int32_t avl_idx_t;

struct avl {
	struct avl *left, *right;
	u_int32_t key;
	int balance;

	avl_idx_t idx;

	size_t count;
	struct avl *next;
};

struct avl_make {
	int fd;
	struct mpool rec;
	struct avl *root;
	size_t count;
	avl_ofs_t cur;
	struct fbuf buf;
};

/* Our on-disk structures */

#define AVL_MAGIC 0xa31a31ff

struct avl_hdr {
	u_int32_t magic;
	u_int32_t count;
};

struct avl_disk {
	u_int32_t key;
	avl_ofs_t ofs;
	avl_ofs_t len;
};

/* Database creation API */
struct avl_make *avl_new_db(int);
int avl_insert(struct avl_make *, u_int32_t, avl_idx_t);
int avl_commit(struct avl_make *);
int avl_abort(struct avl_make *);

/* Database query object */
struct avl_db {
	char	*map;
	char	*end;
	size_t	maplen;
	size_t	count;
	size_t	first,last;
	struct avl_disk *r;
};

/* Database query object */
struct avl_db *avl_open(int);
int avl_select(struct avl_db *, unsigned int, unsigned int);
void avl_close(struct avl_db *);

#endif /* __AVL_HEADER_INCLUDED__ */
