/*
 * This file is part of tcpidx
 * Copyright (c) 2003 Gianni Tedesco
 * Released under the terms of the GNU GPL version 2
 *
 * mem_pool - This is a memory allocator designed for allocating very
 * large numbers of same-sized objects. It has almost zero memory
 * overhead, zero internal fragmentation and exceptionally low external
 * fragmentation.
 *
 * Although it is technically possible to extend the code to free up
 * individual objects, no effort is made to minimise internal
 * fragmentation for this case.
*/
#ifndef __MPOOL_HEADER_INCLUDED__
#define __MPOOL_HEADER_INCLUDED__

struct mpool {
	size_t obj_size;
	size_t slab_size;
	struct mpool_hdr *slabs;
	size_t align;
};

struct mpool_hdr {
	struct mpool_hdr *next;
	void *next_obj; /* next avail. object */
	char data[0];
};

/* Public API */
int mpool_new(struct mpool *m, size_t obj_size, size_t slab_size);
void mpool_free(struct mpool *m);

/* Private API */
void *mpool_alloc_slow(struct mpool *m);

/* Fast path public API */
static inline void *mpool_alloc(struct mpool *m)
{
	if ( m->slabs!=NULL ) {
		void *ret = m->slabs->next_obj;
		m->slabs->next_obj += m->obj_size;

		if ( m->slabs->next_obj <= ((void *)m->slabs) + m->slab_size )
			return ret;
	}

	return mpool_alloc_slow(m);
}

#endif /* __MPOOL_HEADER_INCLUDED__ */
