/*
 * This file is part of SkunkDB
 * Copyright (c) 2002 Gianni Tedesco
 * Released under the terms of the GNU GPL version 2
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <unistd.h>

#include <mpool.h>
#include <fbuf.h>
#include <avl.h>

int read_input(void *sdb, char *map, char *end)
{
	unsigned long long int kval, ival;
	char *cur;
	int state=0;
	char *key=NULL, *val=NULL;
	size_t klen=0, vlen=0;
	char *kend;
	
	for (cur=map; cur < end; cur++) {
		switch ( state ) {
		case 0:
			key=cur;
			val=NULL;
			klen=0;
			vlen=0;
			state=1;
			break;
		case 1:
			if ( *cur == ':' ) {
				state=2;
				val=++cur;
			}

			klen++;
			break;
		case 2:
			vlen++;

			if ( *cur != '\n' )
				break;

			/* convert to integer */
			kval=strtoul(key, &kend, 10);
			if ( kend==key ) {
				fprintf(stderr, "fucked key!\n");
				break;
			}

			ival=strtoul(val, &kend, 10);
			if ( kend==val ) {
				fprintf(stderr, "fucked val!\n");
				break;
			}

			if ( avl_insert(sdb, (u_int32_t)kval, (avl_idx_t)ival)<0 )
				return 0;

			state=0;
			break;
		}
	}

	return 1;
}

int main (int argc, char **argv) 
{
	void *sdb;
	struct stat st;
	char *map;

	/* Find size of input file */
	if ( (fstat(STDIN_FILENO, &st)) < 0 ) {
		perror("fstat");
		return 1;
	}

	/* Memory map input file */
	if ( !(map=mmap(NULL, st.st_size, PROT_READ,
		MAP_SHARED, STDIN_FILENO, 0)) ) {
		perror("mmap");
		return 1;
	}

	/* Make the DB */
	if ( !(sdb=avl_new_db(STDOUT_FILENO)) ) {
		fprintf(stderr, "make_init() failed\n");
		return 1;
	}
	
	/* Read the data */
	fprintf(stderr, "Reading input...\n");
	if ( !read_input(sdb, map, map+st.st_size) ) {
		fprintf(stderr, "read_input() failed\n");
		return 1;
	}
	
	/* Write the DB out */
	fprintf(stderr, "Commit...\n");
	if ( avl_commit(sdb)<0 ) {
		perror("avl_commit()");
		return 1;
	}

	return 0;
}
