/*
 * This file is part of SkunkDB
 * Copyright (c) 2002 Gianni Tedesco
 * Released under the terms of the GNU GPL version 2
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <mpool.h>
#include <skunk.h>

#undef BENCHMARK

#ifdef BENCHMARK
/* Could be faster... */
static inline int itoa(char *buf, int d)
{
	char *p = buf;
	char *p1, *p2;
	unsigned long ud = d;
	int ret=1;
  
	/* Divide UD by DIVISOR until UD == 0.  */
	do
	{
		int remainder = ud % 10;
		*p++ = remainder + '0';
	}while (ud /= 10);

	/* Terminate BUF.  */
	*p = 0;
  
	/* Reverse BUF.  */
	p1 = buf;
	p2 = p - 1;
	while (p1 < p2)
	{
		char tmp = *p1;
		*p1 = *p2;
		*p2 = tmp;
		p1++;
		p2--;
		ret++;
	}
	return ret;
}

void do_benchmark(void *q)
{
	char b[128];
	int i,f=0;
	int ret,x;
	struct rec r;

	skunk_set_result(q, &r);

	for ( i=0; i<1000000; i++ ) {
		x=itoa(b, i);
		if ( !(ret=skunk_select(q, b, x) ) ) {
			f++;
		}else if ( ret==-1 ) {
			printf("DB is corrupt! %i failed\n", f);
			return;
		}
	}
	printf("%i failed\n", f);
}
#endif

int main (int argc, char **argv) 
{
	void *db;
	struct rec r;
	int ret=0;

	if ( !(db=skunk_open(STDIN_FILENO)) ) {
		perror("skunk_open()");
		return 1;
	}

#ifdef BENCHMARK
	do_benchmark(db);
#else
	if ( argc!=2 ) {
		fprintf(stderr, "Usage: %s [key] < dbfile\n", argv[0]);
		return 1;
	}

	skunk_set_result(db, &r);
	switch ( skunk_select(db, argv[1], strlen(argv[1])) ) {
	case 0:
		fprintf(stderr, "%s: key '%s' not found\n", argv[0], argv[1]);
		ret=1;
		break;
	case -1:
		fprintf(stderr, "%s: fatal error: database corrupt\n", argv[0]);
		ret=1;
		break;
	case 1:
		fwrite(r.key, r.key_len, 1, stdout);
		fwrite(":", 1, 1, stdout);
		fwrite(r.val, r.val_len, 1, stdout);
		fwrite("\n", 1, 1, stdout);
		ret=0;
		break;
	}
#endif

	skunk_close(db);
	return ret;
}
